@@ -39,6 +39,19 @@ from .collections import ImmutableDict
 from .hashutil import git_object_header, hash_to_bytehex
 
 
+def content_git_object(content: model.Content) -> bytes:
+    """Formats a content as a git blob.
+
+    A content's identifier is the blob sha1 à la git of the tagged content.
+    """
+    content = cast(model.Content, content)
+
+    if content.data is None:
+        raise model.MissingData("Content data is None, cannot format.")
+
+    return git_object_header("blob", len(content.data)) + content.data
+
+
 def directory_entry_sort_key(entry: model.DirectoryEntry):
     """The sorting key for tree entries"""
     if isinstance(entry, dict):
@@ -96,29 +109,6 @@ def format_date(date: model.Timestamp) -> bytes:
         return float_value.rstrip("0").encode()
 
 
-@lru_cache()
-def format_offset(offset: int, negative_utc: Optional[bool] = None) -> bytes:
-    """Convert an integer number of minutes into an offset representation.
-
-    The offset representation is [+-]hhmm where:
-
-    - hh is the number of hours;
-    - mm is the number of minutes.
-
-    A null offset is represented as +0000.
-    """
-    if offset < 0 or offset == 0 and negative_utc:
-        sign = "-"
-    else:
-        sign = "+"
-
-    hours = abs(offset) // 60
-    minutes = abs(offset) % 60
-
-    t = "%s%02d%02d" % (sign, hours, minutes)
-    return t.encode()
-
-
 def normalize_timestamp(time_representation):
     """Normalize a time representation for processing by Software Heritage
 
@@ -200,7 +190,13 @@ def directory_git_object(directory: Union[Dict, model.Directory]) -> bytes:
 
     for entry in sorted(directory.entries, key=directory_entry_sort_key):
         components.extend(
-            [_perms_to_bytes(entry.perms), b"\x20", entry.name, b"\x00", entry.target,]
+            [
+                _perms_to_bytes(entry.perms),
+                b"\x20",
+                entry.name,
+                b"\x00",
+                entry.target,
+            ]
         )
 
     return format_git_object_from_parts("tree", components)
@@ -244,10 +240,7 @@ def format_git_object_from_headers(
     if message is not None:
         entries.extend((b"\n", message))
 
-    concatenated_entries = b"".join(entries)
-
-    header = git_object_header(git_type, len(concatenated_entries))
-    return header + concatenated_entries
+    return format_git_object_from_parts(git_type, entries)
 
 
 def format_git_object_from_parts(git_type: str, parts: Iterable[bytes]) -> bytes:
@@ -291,9 +284,8 @@ def format_author_data(
 
     if date_offset is not None:
         date_f = format_date(date_offset.timestamp)
-        offset_f = format_offset(date_offset.offset, date_offset.negative_utc)
 
-        ret.extend([b" ", date_f, b" ", offset_f])
+        ret.extend([b" ", date_f, b" ", date_offset.offset_bytes])
 
     return b"".join(ret)
 
@@ -364,10 +356,15 @@ def revision_git_object(revision: Union[Dict, model.Revision]) -> bytes:
         if parent:
             headers.append((b"parent", hash_to_bytehex(parent)))
 
-    headers.append((b"author", format_author_data(revision.author, revision.date)))
-    headers.append(
-        (b"committer", format_author_data(revision.committer, revision.committer_date),)
-    )
+    if revision.author is not None:
+        headers.append((b"author", format_author_data(revision.author, revision.date)))
+    if revision.committer is not None:
+        headers.append(
+            (
+                b"committer",
+                format_author_data(revision.committer, revision.committer_date),
+            )
+        )
 
     # Handle extra headers
     metadata = revision.metadata or ImmutableDict()
@@ -415,7 +412,9 @@ def release_git_object(release: Union[Dict, model.Release]) -> bytes:
     return format_git_object_from_headers("tag", headers, release.message)
 
 
-def snapshot_git_object(snapshot: Union[Dict, model.Snapshot]) -> bytes:
+def snapshot_git_object(
+    snapshot: Union[Dict, model.Snapshot], *, ignore_unresolved: bool = False
+) -> bytes:
     """Formats a snapshot as a git-like object.
 
     Snapshots are a set of named branches, which are pointers to objects at any
@@ -459,6 +458,10 @@ def snapshot_git_object(snapshot: Union[Dict, model.Snapshot]) -> bytes:
       Note that, akin to directory manifests, there is no separator between
       entries. Because of symbolic branches, identifiers are of arbitrary
       length but are length-encoded to avoid ambiguity.
+
+    Args:
+      ignore_unresolved: if False (the default), raises an exception when
+        alias branches point to non-existing branches
     """
     if isinstance(snapshot, dict):
         # For backward compatibility
@@ -498,7 +501,7 @@ def snapshot_git_object(snapshot: Union[Dict, model.Snapshot]) -> bytes:
             ]
         )
 
-    if unresolved:
+    if unresolved and not ignore_unresolved:
         raise ValueError(
             "Branch aliases unresolved: %s"
             % ", ".join("%r -> %r" % x for x in unresolved),
@@ -583,7 +586,10 @@ def raw_extrinsic_metadata_git_object(
             b"authority",
             f"{metadata.authority.type.value} {metadata.authority.url}".encode(),
         ),
-        (b"fetcher", f"{metadata.fetcher.name} {metadata.fetcher.version}".encode(),),
+        (
+            b"fetcher",
+            f"{metadata.fetcher.name} {metadata.fetcher.version}".encode(),
+        ),
         (b"format", metadata.format.encode()),
     ]
 
@@ -639,7 +645,10 @@ def extid_git_object(extid: model.ExtID) -> bytes:
         headers.append((b"extid_version", str(extid_version).encode("ascii")))
 
     headers.extend(
-        [(b"extid", extid.extid), (b"target", str(extid.target).encode("ascii")),]
+        [
+            (b"extid", extid.extid),
+            (b"target", str(extid.target).encode("ascii")),
+        ]
     )
 
     return format_git_object_from_headers("extid", headers)
