@@ -16,10 +16,11 @@ method to convert between them and msgpack-serializable objects.
 """
 
 from abc import ABCMeta, abstractmethod
+import collections
 import datetime
 from enum import Enum
 import hashlib
-from typing import Any, Dict, Iterable, Optional, Tuple, TypeVar, Union
+from typing import Any, Dict, Iterable, List, Optional, Tuple, Type, TypeVar, Union
 
 import attr
 from attrs_strict import AttributeTypeError
@@ -29,7 +30,7 @@ from typing_extensions import Final
 
 from . import git_objects
 from .collections import ImmutableDict
-from .hashutil import DEFAULT_ALGORITHMS, MultiHash, hash_to_hex
+from .hashutil import DEFAULT_ALGORITHMS, MultiHash, hash_to_bytehex, hash_to_hex
 from .swhids import CoreSWHID
 from .swhids import ExtendedObjectType as SwhidExtendedObjectType
 from .swhids import ExtendedSWHID
@@ -196,7 +197,9 @@ class BaseModel:
 
     def check(self) -> None:
         """Performs internal consistency checks, and raises an error if one fails."""
-        attr.validate(self)
+        # without the type-ignore comment below, attr >= 22.1.0 causes mypy to report:
+        #   Argument 1 has incompatible type "BaseModel"; expected "AttrsInstance"
+        attr.validate(self)  # type: ignore[arg-type]
 
 
 def _compute_hash_from_manifest(manifest: bytes) -> Sha1Git:
@@ -243,6 +246,8 @@ class HashableObjectWithManifest(HashableObject):
     """Derived class of HashableObject, for objects that may need to store
     verbatim git objects as ``raw_manifest`` to preserve original hashes."""
 
+    __slots__ = ()
+
     raw_manifest: Optional[bytes] = None
     """Stores the original content of git objects when they cannot be faithfully
     represented using only the other attributes.
@@ -264,7 +269,7 @@ class HashableObjectWithManifest(HashableObject):
         attribute is set to an empty value.
         """
         if self.raw_manifest is None:
-            return super().compute_hash()
+            return super().compute_hash()  # calls self._compute_hash_from_attributes()
         else:
             return _compute_hash_from_manifest(self.raw_manifest)
 
@@ -287,8 +292,8 @@ class Person(BaseModel):
     object_type: Final = "person"
 
     fullname = attr.ib(type=bytes, validator=type_validator())
-    name = attr.ib(type=Optional[bytes], validator=type_validator())
-    email = attr.ib(type=Optional[bytes], validator=type_validator())
+    name = attr.ib(type=Optional[bytes], validator=type_validator(), eq=False)
+    email = attr.ib(type=Optional[bytes], validator=type_validator(), eq=False)
 
     @classmethod
     def from_fullname(cls, fullname: bytes):
@@ -323,7 +328,11 @@ class Person(BaseModel):
             else:
                 email = raw_email[:close_bracket]
 
-        return Person(name=name or None, email=email or None, fullname=fullname,)
+        return Person(
+            name=name or None,
+            email=email or None,
+            fullname=fullname,
+        )
 
     def anonymize(self) -> "Person":
         """Returns an anonymized version of the Person object.
@@ -332,7 +341,9 @@ class Person(BaseModel):
         or email.
         """
         return Person(
-            fullname=hashlib.sha256(self.fullname).digest(), name=None, email=None,
+            fullname=hashlib.sha256(self.fullname).digest(),
+            name=None,
+            email=None,
         )
 
     @classmethod
@@ -368,13 +379,13 @@ class Timestamp(BaseModel):
     @seconds.validator
     def check_seconds(self, attribute, value):
         """Check that seconds fit in a 64-bits signed integer."""
-        if not (-(2 ** 63) <= value < 2 ** 63):
+        if not (-(2**63) <= value < 2**63):
             raise ValueError("Seconds must be a signed 64-bits integer.")
 
     @microseconds.validator
     def check_microseconds(self, attribute, value):
         """Checks that microseconds are positive and < 1000000."""
-        if not (0 <= value < 10 ** 6):
+        if not (0 <= value < 10**6):
             raise ValueError("Microseconds must be in [0, 1000000[.")
 
 
@@ -385,8 +396,6 @@ class TimestampWithTimezone(BaseModel):
     object_type: Final = "timestamp_with_timezone"
 
     timestamp = attr.ib(type=Timestamp, validator=type_validator())
-    offset = attr.ib(type=int, validator=type_validator())
-    negative_utc = attr.ib(type=bool, validator=type_validator())
 
     offset_bytes = attr.ib(type=bytes, validator=type_validator())
     """Raw git representation of the timezone, as an offset from UTC.
@@ -397,64 +406,28 @@ class TimestampWithTimezone(BaseModel):
     original objects, so it may differ from this format when they do.
     """
 
-    @offset.validator
-    def check_offset(self, attribute, value):
-        """Checks the offset is a 16-bits signed integer (in theory, it
-        should always be between -14 and +14 hours)."""
-        if not (-(2 ** 15) <= value < 2 ** 15):
-            # max 14 hours offset in theory, but you never know what
-            # you'll find in the wild...
-            raise ValueError("offset too large: %d minutes" % value)
-
-        self._check_offsets_match()
-
-    @negative_utc.validator
-    def check_negative_utc(self, attribute, value):
-        if self.offset and value:
-            raise ValueError("negative_utc can only be True is offset=0")
-
-        self._check_offsets_match()
-
-    @offset_bytes.default
-    def _default_offset_bytes(self):
-        negative = self.offset < 0 or self.negative_utc
-        (hours, minutes) = divmod(abs(self.offset), 60)
-        return f"{'-' if negative else '+'}{hours:02}{minutes:02}".encode()
-
-    @offset_bytes.validator
-    def check_offset_bytes(self, attribute, value):
-        if not set(value) <= _OFFSET_CHARS:
-            raise ValueError(f"invalid characters in offset_bytes: {value!r}")
-
-        self._check_offsets_match()
-
-    def _check_offsets_match(self):
-        offset_str = self.offset_bytes.decode()
-        assert offset_str[0] in "+-"
-        sign = int(offset_str[0] + "1")
-        hours = int(offset_str[1:-2])
-        minutes = int(offset_str[-2:])
-        offset = sign * (hours * 60 + minutes)
-        if offset != self.offset:
-            raise ValueError(
-                f"offset_bytes ({self.offset_bytes!r}) does not match offset "
-                f"{divmod(self.offset, 60)}"
-            )
-
-        if offset == 0 and self.negative_utc != self.offset_bytes.startswith(b"-"):
-            raise ValueError(
-                f"offset_bytes ({self.offset_bytes!r}) does not match negative_utc "
-                f"({self.negative_utc})"
-            )
+    @classmethod
+    def from_numeric_offset(
+        cls, timestamp: Timestamp, offset: int, negative_utc: bool
+    ) -> "TimestampWithTimezone":
+        """Returns a :class:`TimestampWithTimezone` instance from the old dictionary
+        format (with ``offset`` and ``negative_utc`` instead of ``offset_bytes``).
+        """
+        negative = offset < 0 or negative_utc
+        (hours, minutes) = divmod(abs(offset), 60)
+        offset_bytes = f"{'-' if negative else '+'}{hours:02}{minutes:02}".encode()
+        tstz = TimestampWithTimezone(timestamp=timestamp, offset_bytes=offset_bytes)
+        assert tstz.offset_minutes() == offset, (tstz.offset_minutes(), offset)
+        return tstz
 
     @classmethod
-    def from_dict(cls, time_representation: Union[Dict, datetime.datetime, int]):
+    def from_dict(
+        cls, time_representation: Union[Dict, datetime.datetime, int]
+    ) -> "TimestampWithTimezone":
         """Builds a TimestampWithTimezone from any of the formats
         accepted by :func:`swh.model.normalize_timestamp`."""
         # TODO: this accept way more types than just dicts; find a better
         # name
-        negative_utc = False
-
         if isinstance(time_representation, dict):
             ts = time_representation["timestamp"]
             if isinstance(ts, dict):
@@ -468,17 +441,27 @@ class TimestampWithTimezone(BaseModel):
                     f"TimestampWithTimezone.from_dict received non-integer timestamp "
                     f"member {ts!r}"
                 )
-            offset = time_representation["offset"]
-            if "negative_utc" in time_representation:
-                negative_utc = time_representation["negative_utc"]
-            if negative_utc is None:
-                negative_utc = False
+
+            timestamp = Timestamp(seconds=seconds, microseconds=microseconds)
+
+            if "offset_bytes" in time_representation:
+                return cls(
+                    timestamp=timestamp,
+                    offset_bytes=time_representation["offset_bytes"],
+                )
+            else:
+                # old format
+                offset = time_representation["offset"]
+                negative_utc = time_representation.get("negative_utc") or False
+                return cls.from_numeric_offset(timestamp, offset, negative_utc)
         elif isinstance(time_representation, datetime.datetime):
+            # TODO: warn when using from_dict() on a datetime
+            utcoffset = time_representation.utcoffset()
+            time_representation = time_representation.astimezone(datetime.timezone.utc)
             microseconds = time_representation.microsecond
             if microseconds:
                 time_representation = time_representation.replace(microsecond=0)
             seconds = int(time_representation.timestamp())
-            utcoffset = time_representation.utcoffset()
             if utcoffset is None:
                 raise ValueError(
                     f"TimestampWithTimezone.from_dict received datetime without "
@@ -488,52 +471,120 @@ class TimestampWithTimezone(BaseModel):
             # utcoffset is an integer number of minutes
             seconds_offset = utcoffset.total_seconds()
             offset = int(seconds_offset) // 60
+            # TODO: warn if remainder is not zero
+            return cls.from_numeric_offset(
+                Timestamp(seconds=seconds, microseconds=microseconds), offset, False
+            )
         elif isinstance(time_representation, int):
+            # TODO: warn when using from_dict() on an int
             seconds = time_representation
-            microseconds = 0
-            offset = 0
+            timestamp = Timestamp(seconds=time_representation, microseconds=0)
+            return cls(timestamp=timestamp, offset_bytes=b"+0000")
         else:
             raise ValueError(
                 f"TimestampWithTimezone.from_dict received non-integer timestamp: "
                 f"{time_representation!r}"
             )
 
-        return cls(
-            timestamp=Timestamp(seconds=seconds, microseconds=microseconds),
-            offset=offset,
-            negative_utc=negative_utc,
-        )
-
     @classmethod
-    def from_datetime(cls, dt: datetime.datetime):
+    def from_datetime(cls, dt: datetime.datetime) -> "TimestampWithTimezone":
         return cls.from_dict(dt)
 
     def to_datetime(self) -> datetime.datetime:
         """Convert to a datetime (with a timezone set to the recorded fixed UTC offset)
 
-        Beware that this conversion can be lossy: the negative_utc flag is not
-        taken into consideration (since it cannot be represented in a
-        datetime). Also note that it may fail due to type overflow.
-
+        Beware that this conversion can be lossy: ``-0000`` and 'weird' offsets
+        cannot be represented. Also note that it may fail due to type overflow.
         """
         timestamp = datetime.datetime.fromtimestamp(
             self.timestamp.seconds,
-            datetime.timezone(datetime.timedelta(minutes=self.offset)),
+            datetime.timezone(datetime.timedelta(minutes=self.offset_minutes())),
         )
         timestamp = timestamp.replace(microsecond=self.timestamp.microseconds)
         return timestamp
 
     @classmethod
     def from_iso8601(cls, s):
-        """Builds a TimestampWithTimezone from an ISO8601-formatted string.
-        """
+        """Builds a TimestampWithTimezone from an ISO8601-formatted string."""
         dt = iso8601.parse_date(s)
         tstz = cls.from_datetime(dt)
         if dt.tzname() == "-00:00":
             assert tstz.offset_bytes == b"+0000"
-            tstz = attr.evolve(tstz, negative_utc=True, offset_bytes=b"-0000")
+            tstz = attr.evolve(tstz, offset_bytes=b"-0000")
         return tstz
 
+    @staticmethod
+    def _parse_offset_bytes(offset_bytes: bytes) -> int:
+        """Parses an ``offset_bytes`` value (in Git's ``[+-]HHMM`` format),
+        and returns the corresponding numeric values (in number of minutes).
+
+        Tries to account for some mistakes in the format, to support incorrect
+        Git implementations.
+
+        >>> TimestampWithTimezone._parse_offset_bytes(b"+0000")
+        0
+        >>> TimestampWithTimezone._parse_offset_bytes(b"-0000")
+        0
+        >>> TimestampWithTimezone._parse_offset_bytes(b"+0200")
+        120
+        >>> TimestampWithTimezone._parse_offset_bytes(b"-0200")
+        -120
+        >>> TimestampWithTimezone._parse_offset_bytes(b"+200")
+        120
+        >>> TimestampWithTimezone._parse_offset_bytes(b"-200")
+        -120
+        >>> TimestampWithTimezone._parse_offset_bytes(b"+02")
+        120
+        >>> TimestampWithTimezone._parse_offset_bytes(b"-02")
+        -120
+        >>> TimestampWithTimezone._parse_offset_bytes(b"+0010")
+        10
+        >>> TimestampWithTimezone._parse_offset_bytes(b"-0010")
+        -10
+        >>> TimestampWithTimezone._parse_offset_bytes(b"+200000000000000000")
+        0
+        >>> TimestampWithTimezone._parse_offset_bytes(b"+0160")  # 60 minutes...
+        0
+        """
+        offset_str = offset_bytes.decode()
+        assert offset_str[0] in "+-"
+        sign = int(offset_str[0] + "1")
+        if len(offset_str) <= 3:
+            hours = int(offset_str[1:])
+            minutes = 0
+        else:
+            hours = int(offset_str[1:-2])
+            minutes = int(offset_str[-2:])
+
+        offset = sign * (hours * 60 + minutes)
+        if (0 <= minutes <= 59) and (-(2**15) <= offset < 2**15):
+            return offset
+        else:
+            # can't parse it to a reasonable value; give up and pretend it's UTC.
+            return 0
+
+    def offset_minutes(self):
+        """Returns the offset, as a number of minutes since UTC.
+
+        >>> TimestampWithTimezone(
+        ...     Timestamp(seconds=1642765364, microseconds=0), offset_bytes=b"+0000"
+        ... ).offset_minutes()
+        0
+        >>> TimestampWithTimezone(
+        ...     Timestamp(seconds=1642765364, microseconds=0), offset_bytes=b"+0200"
+        ... ).offset_minutes()
+        120
+        >>> TimestampWithTimezone(
+        ...     Timestamp(seconds=1642765364, microseconds=0), offset_bytes=b"-0200"
+        ... ).offset_minutes()
+        -120
+        >>> TimestampWithTimezone(
+        ...     Timestamp(seconds=1642765364, microseconds=0), offset_bytes=b"+0530"
+        ... ).offset_minutes()
+        330
+        """
+        return self._parse_offset_bytes(self.offset_bytes)
+
 
 @attr.s(frozen=True, slots=True)
 class Origin(HashableObject, BaseModel):
@@ -554,7 +605,8 @@ class Origin(HashableObject, BaseModel):
     def swhid(self) -> ExtendedSWHID:
         """Returns a SWHID representing this origin."""
         return ExtendedSWHID(
-            object_type=SwhidExtendedObjectType.ORIGIN, object_id=self.id,
+            object_type=SwhidExtendedObjectType.ORIGIN,
+            object_id=self.id,
         )
 
 
@@ -591,9 +643,7 @@ class OriginVisit(BaseModel):
 
 @attr.s(frozen=True, slots=True)
 class OriginVisitStatus(BaseModel):
-    """Represents a visit update of an origin at a given point in time.
-
-    """
+    """Represents a visit update of an origin at a given point in time."""
 
     object_type: Final = "origin_visit_status"
 
@@ -693,7 +743,9 @@ class Snapshot(HashableObject, BaseModel):
     id = attr.ib(type=Sha1Git, validator=type_validator(), default=b"", repr=hash_repr)
 
     def _compute_hash_from_attributes(self) -> bytes:
-        return _compute_hash_from_manifest(git_objects.snapshot_git_object(self))
+        return _compute_hash_from_manifest(
+            git_objects.snapshot_git_object(self, ignore_unresolved=True)
+        )
 
     @classmethod
     def from_dict(cls, d):
@@ -792,8 +844,8 @@ class Revision(HashableObjectWithManifest, BaseModel):
     object_type: Final = "revision"
 
     message = attr.ib(type=Optional[bytes], validator=type_validator())
-    author = attr.ib(type=Person, validator=type_validator())
-    committer = attr.ib(type=Person, validator=type_validator())
+    author = attr.ib(type=Optional[Person], validator=type_validator())
+    committer = attr.ib(type=Optional[Person], validator=type_validator())
     date = attr.ib(type=Optional[TimestampWithTimezone], validator=type_validator())
     committer_date = attr.ib(
         type=Optional[TimestampWithTimezone], validator=type_validator()
@@ -826,7 +878,9 @@ class Revision(HashableObjectWithManifest, BaseModel):
             if not self.extra_headers and "extra_headers" in metadata:
                 (extra_headers, metadata) = metadata.copy_pop("extra_headers")
                 object.__setattr__(
-                    self, "extra_headers", tuplify_extra_headers(extra_headers),
+                    self,
+                    "extra_headers",
+                    tuplify_extra_headers(extra_headers),
                 )
                 attr.validate(self)
             object.__setattr__(self, "metadata", metadata)
@@ -834,6 +888,20 @@ class Revision(HashableObjectWithManifest, BaseModel):
     def _compute_hash_from_attributes(self) -> bytes:
         return _compute_hash_from_manifest(git_objects.revision_git_object(self))
 
+    @author.validator
+    def check_author(self, attribute, value):
+        """If the author is `None`, checks the date is `None` too."""
+        if self.author is None and self.date is not None:
+            raise ValueError("revision date must be None if author is None.")
+
+    @committer.validator
+    def check_committer(self, attribute, value):
+        """If the committer is `None`, checks the committer_date is `None` too."""
+        if self.committer is None and self.committer_date is not None:
+            raise ValueError(
+                "revision committer_date must be None if committer is None."
+            )
+
     @classmethod
     def from_dict(cls, d):
         d = d.copy()
@@ -845,9 +913,17 @@ class Revision(HashableObjectWithManifest, BaseModel):
         if committer_date:
             committer_date = TimestampWithTimezone.from_dict(committer_date)
 
+        author = d.pop("author")
+        if author:
+            author = Person.from_dict(author)
+
+        committer = d.pop("committer")
+        if committer:
+            committer = Person.from_dict(committer)
+
         return cls(
-            author=Person.from_dict(d.pop("author")),
-            committer=Person.from_dict(d.pop("committer")),
+            author=author,
+            committer=committer,
             date=date,
             committer_date=committer_date,
             type=RevisionType(d.pop("type")),
@@ -866,16 +942,21 @@ class Revision(HashableObjectWithManifest, BaseModel):
         Person object.
         """
         return attr.evolve(
-            self, author=self.author.anonymize(), committer=self.committer.anonymize()
+            self,
+            author=None if self.author is None else self.author.anonymize(),
+            committer=None if self.committer is None else self.committer.anonymize(),
         )
 
 
+_DIR_ENTRY_TYPES = ["file", "dir", "rev"]
+
+
 @attr.s(frozen=True, slots=True)
 class DirectoryEntry(BaseModel):
     object_type: Final = "directory_entry"
 
     name = attr.ib(type=bytes, validator=type_validator())
-    type = attr.ib(type=str, validator=attr.validators.in_(["file", "dir", "rev"]))
+    type = attr.ib(type=str, validator=attr.validators.in_(_DIR_ENTRY_TYPES))
     target = attr.ib(type=Sha1Git, validator=type_validator(), repr=hash_repr)
     perms = attr.ib(type=int, validator=type_validator(), converter=int, repr=oct)
     """Usually one of the values of `swh.model.from_disk.DentryPerms`."""
@@ -902,8 +983,10 @@ class Directory(HashableObjectWithManifest, BaseModel):
         seen = set()
         for entry in value:
             if entry.name in seen:
+                # Cannot use self.swhid() here, self.id may be None
                 raise ValueError(
-                    "{self.swhid()} has duplicated entry name: {entry.name!r}"
+                    f"swh:1:dir:{hash_to_hex(self.id)} has duplicated entry name: "
+                    f"{entry.name!r}"
                 )
             seen.add(entry.name)
 
@@ -921,6 +1004,87 @@ class Directory(HashableObjectWithManifest, BaseModel):
         """Returns a SWHID representing this object."""
         return CoreSWHID(object_type=SwhidObjectType.DIRECTORY, object_id=self.id)
 
+    @classmethod
+    def from_possibly_duplicated_entries(
+        cls,
+        *,
+        entries: Tuple[DirectoryEntry, ...],
+        id: Sha1Git = b"",
+        raw_manifest: Optional[bytes] = None,
+    ) -> Tuple[bool, "Directory"]:
+        """Constructs a ``Directory`` object from a list of entries that may contain
+        duplicated names.
+
+        This is required to represent legacy objects, that were ingested in the
+        storage database before this check was added.
+
+        As it is impossible for a ``Directory`` instances to have more than one entry
+        with a given names, this function computes a ``raw_manifest`` and renames one of
+        the entries before constructing the ``Directory``.
+
+        Returns:
+            ``(is_corrupt, directory)`` where ``is_corrupt`` is True iff some
+            entry names were indeed duplicated
+        """
+        # First, try building a Directory object normally without any extra computation,
+        # which works the overwhelming majority of the time:
+        try:
+            return (False, Directory(entries=entries, id=id, raw_manifest=raw_manifest))
+        except ValueError:
+            pass
+
+        # If it fails:
+        # 1. compute a raw_manifest if there isn't already one:
+        if raw_manifest is None:
+            # invalid_directory behaves like a Directory object, but without the
+            # duplicated entry check; which allows computing its raw_manifest
+            invalid_directory = type("", (), {})()
+            invalid_directory.entries = entries
+            raw_manifest = git_objects.directory_git_object(invalid_directory)
+
+        # 2. look for duplicated entries:
+        entries_by_name: Dict[
+            bytes, Dict[str, List[DirectoryEntry]]
+        ] = collections.defaultdict(lambda: collections.defaultdict(list))
+        for entry in entries:
+            entries_by_name[entry.name][entry.type].append(entry)
+
+        # 3. strip duplicates
+        deduplicated_entries = []
+        for entry_lists in entries_by_name.values():
+            # We could pick one entry at random to keep the original name; but we try to
+            # "minimize" the impact, by preserving entries of type "rev" first
+            # (because renaming them would likely break git submodules entirely
+            # when this directory is written to disk),
+            # then entries of type "dir" (because renaming them affects the path
+            # of every file in the dir, instead of just one "cnt").
+            dir_entry_types = ("rev", "dir", "file")
+            assert set(dir_entry_types) == set(_DIR_ENTRY_TYPES)
+            picked_winner = False  # when True, all future entries must be renamed
+            for type_ in dir_entry_types:
+                for entry in entry_lists[type_]:
+                    if not picked_winner:
+                        # this is the "most important" entry according to this
+                        # heuristic; it gets to keep its name.
+                        deduplicated_entries.append(entry)
+                        picked_winner = True
+                    else:
+                        # the heuristic already found an entry more important than
+                        # this one; so this one must be renamed to something.
+                        # we pick the beginning of its hash, it should be good enough
+                        # to avoid any conflict.
+                        new_name = (
+                            entry.name + b"_" + hash_to_bytehex(entry.target)[0:10]
+                        )
+                        renamed_entry = attr.evolve(entry, name=new_name)
+                        deduplicated_entries.append(renamed_entry)
+
+        # Finally, return the "fixed" the directory
+        dir_ = Directory(
+            entries=tuple(deduplicated_entries), id=id, raw_manifest=raw_manifest
+        )
+        return (True, dir_)
+
 
 @attr.s(frozen=True, slots=True)
 class BaseContent(BaseModel):
@@ -1400,6 +1564,12 @@ class RawExtrinsicMetadata(HashableObject, BaseModel):
 
     @classmethod
     def from_dict(cls, d):
+        if "type" in d:
+            # Convert from old schema
+            type_ = d.pop("type")
+            if type_ == "origin":
+                d["target"] = str(Origin(d["target"]).swhid())
+
         d = {
             **d,
             "target": ExtendedSWHID.from_string(d["target"]),
@@ -1444,3 +1614,31 @@ class ExtID(HashableObject, BaseModel):
 
     def _compute_hash_from_attributes(self) -> bytes:
         return _compute_hash_from_manifest(git_objects.extid_git_object(self))
+
+
+# Note: we need the type ignore stanza here because mypy cannot figure that all
+# subclasses of BaseModel do have an object_type attribute, even if BaseModel
+# itself does not (because these are Final)
+SWH_MODEL_OBJECT_TYPES: Dict[str, Type[BaseModel]] = {
+    cls.object_type: cls  # type: ignore
+    for cls in (
+        Person,
+        Timestamp,
+        TimestampWithTimezone,
+        Origin,
+        OriginVisit,
+        OriginVisitStatus,
+        Snapshot,
+        SnapshotBranch,
+        Release,
+        Revision,
+        Directory,
+        DirectoryEntry,
+        Content,
+        SkippedContent,
+        MetadataAuthority,
+        MetadataFetcher,
+        RawExtrinsicMetadata,
+        ExtID,
+    )
+}
