@@ -13,7 +13,7 @@ import attr
 from attrs_strict import AttributeTypeError
 import dateutil
 from hypothesis import given
-from hypothesis.strategies import binary
+from hypothesis.strategies import binary, none
 import pytest
 
 from swh.model.collections import ImmutableDict
@@ -215,8 +215,16 @@ _TYPE_VALIDATOR_PARAMETERS: List[Tuple[Any, List[Any], List[Any]]] = [
         [ImmutableDict({"foo": "bar"}), ImmutableDict({42: 123})],
     ),
     # Any:
-    (object, [-1, 0, 1, 42, 1000, None, "123", 0.0, (), ImmutableDict()], [],),
-    (Any, [-1, 0, 1, 42, 1000, None, "123", 0.0, (), ImmutableDict()], [],),
+    (
+        object,
+        [-1, 0, 1, 42, 1000, None, "123", 0.0, (), ImmutableDict()],
+        [],
+    ),
+    (
+        Any,
+        [-1, 0, 1, 42, 1000, None, "123", 0.0, (), ImmutableDict()],
+        [],
+    ),
     (
         ImmutableDict[Any, int],
         [
@@ -240,10 +248,16 @@ _TYPE_VALIDATOR_PARAMETERS: List[Tuple[Any, List[Any], List[Any]]] = [
     # attr objects:
     (
         Timestamp,
-        [Timestamp(seconds=123, microseconds=0),],
+        [
+            Timestamp(seconds=123, microseconds=0),
+        ],
         [None, "2021-09-28T11:27:59", 123],
     ),
-    (Cls1, [Cls1(), Cls2()], [None, b"abcd"],),
+    (
+        Cls1,
+        [Cls1(), Cls2()],
+        [None, b"abcd"],
+    ),
     # enums:
     (
         TargetType,
@@ -300,7 +314,11 @@ def test_unique_key():
     }
     assert OriginVisitStatus(
         origin=url, visit=42, date=date, status="created", snapshot=None
-    ).unique_key() == {"origin": url, "visit": "42", "date": str(date),}
+    ).unique_key() == {
+        "origin": url,
+        "visit": "42",
+        "date": str(date),
+    }
 
     assert Snapshot.from_dict({**snapshot_example, "id": id_}).unique_key() == id_
     assert Release.from_dict({**release_example, "id": id_}).unique_key() == id_
@@ -372,7 +390,9 @@ def test_todict_origin_visits(origin_visit):
 def test_origin_visit_naive_datetime():
     with pytest.raises(ValueError, match="must be a timezone-aware datetime"):
         OriginVisit(
-            origin="http://foo/", date=datetime.datetime.now(), type="git",
+            origin="http://foo/",
+            date=datetime.datetime.now(),
+            type="git",
         )
 
 
@@ -407,13 +427,13 @@ def test_timestamp_seconds():
     with pytest.raises(AttributeTypeError):
         Timestamp(seconds="0", microseconds=0)
 
-    attr.validate(Timestamp(seconds=2 ** 63 - 1, microseconds=0))
+    attr.validate(Timestamp(seconds=2**63 - 1, microseconds=0))
     with pytest.raises(ValueError):
-        Timestamp(seconds=2 ** 63, microseconds=0)
+        Timestamp(seconds=2**63, microseconds=0)
 
-    attr.validate(Timestamp(seconds=-(2 ** 63), microseconds=0))
+    attr.validate(Timestamp(seconds=-(2**63), microseconds=0))
     with pytest.raises(ValueError):
-        Timestamp(seconds=-(2 ** 63) - 1, microseconds=0)
+        Timestamp(seconds=-(2**63) - 1, microseconds=0)
 
 
 def test_timestamp_microseconds():
@@ -421,9 +441,9 @@ def test_timestamp_microseconds():
     with pytest.raises(AttributeTypeError):
         Timestamp(seconds=0, microseconds="0")
 
-    attr.validate(Timestamp(seconds=0, microseconds=10 ** 6 - 1))
+    attr.validate(Timestamp(seconds=0, microseconds=10**6 - 1))
     with pytest.raises(ValueError):
-        Timestamp(seconds=0, microseconds=10 ** 6)
+        Timestamp(seconds=0, microseconds=10**6)
 
     with pytest.raises(ValueError):
         Timestamp(seconds=0, microseconds=-1)
@@ -440,9 +460,9 @@ def test_timestamp_from_dict():
     with pytest.raises(ValueError):
         Timestamp.from_dict({"seconds": 0, "microseconds": -1})
 
-    Timestamp.from_dict({"seconds": 0, "microseconds": 10 ** 6 - 1})
+    Timestamp.from_dict({"seconds": 0, "microseconds": 10**6 - 1})
     with pytest.raises(ValueError):
-        Timestamp.from_dict({"seconds": 0, "microseconds": 10 ** 6})
+        Timestamp.from_dict({"seconds": 0, "microseconds": 10**6})
 
 
 # TimestampWithTimezone
@@ -450,55 +470,120 @@ def test_timestamp_from_dict():
 
 def test_timestampwithtimezone():
     ts = Timestamp(seconds=0, microseconds=0)
-    tstz = TimestampWithTimezone(timestamp=ts, offset=0, negative_utc=False)
+    tstz = TimestampWithTimezone(timestamp=ts, offset_bytes=b"+0000")
     attr.validate(tstz)
-    assert tstz.negative_utc is False
+    assert tstz.offset_minutes() == 0
     assert tstz.offset_bytes == b"+0000"
 
-    tstz = TimestampWithTimezone(timestamp=ts, offset=10, negative_utc=False)
+    tstz = TimestampWithTimezone(timestamp=ts, offset_bytes=b"+0010")
     attr.validate(tstz)
+    assert tstz.offset_minutes() == 10
     assert tstz.offset_bytes == b"+0010"
 
-    tstz = TimestampWithTimezone(timestamp=ts, offset=-10, negative_utc=False)
+    tstz = TimestampWithTimezone(timestamp=ts, offset_bytes=b"-0010")
     attr.validate(tstz)
+    assert tstz.offset_minutes() == -10
     assert tstz.offset_bytes == b"-0010"
 
-    tstz = TimestampWithTimezone(timestamp=ts, offset=0, negative_utc=True)
+    tstz = TimestampWithTimezone(timestamp=ts, offset_bytes=b"-0000")
     attr.validate(tstz)
-    assert tstz.negative_utc is True
+    assert tstz.offset_minutes() == 0
     assert tstz.offset_bytes == b"-0000"
 
+    tstz = TimestampWithTimezone(timestamp=ts, offset_bytes=b"-1030")
+    attr.validate(tstz)
+    assert tstz.offset_minutes() == -630
+    assert tstz.offset_bytes == b"-1030"
+
+    tstz = TimestampWithTimezone(timestamp=ts, offset_bytes=b"+1320")
+    attr.validate(tstz)
+    assert tstz.offset_minutes() == 800
+    assert tstz.offset_bytes == b"+1320"
+
+    tstz = TimestampWithTimezone(timestamp=ts, offset_bytes=b"+200")
+    attr.validate(tstz)
+    assert tstz.offset_minutes() == 120
+    assert tstz.offset_bytes == b"+200"
+
+    tstz = TimestampWithTimezone(timestamp=ts, offset_bytes=b"+02")
+    attr.validate(tstz)
+    assert tstz.offset_minutes() == 120
+    assert tstz.offset_bytes == b"+02"
+
+    tstz = TimestampWithTimezone(timestamp=ts, offset_bytes=b"+2000000000")
+    attr.validate(tstz)
+    assert tstz.offset_minutes() == 0
+    assert tstz.offset_bytes == b"+2000000000"
+
     with pytest.raises(AttributeTypeError):
-        TimestampWithTimezone(
-            timestamp=datetime.datetime.now(), offset=0, negative_utc=False
-        )
+        TimestampWithTimezone(timestamp=datetime.datetime.now(), offset_bytes=b"+0000")
 
     with pytest.raises((AttributeTypeError, TypeError)):
-        TimestampWithTimezone(timestamp=ts, offset="0", negative_utc=False)
-
-    with pytest.raises(AttributeTypeError):
-        TimestampWithTimezone(timestamp=ts, offset=1.0, negative_utc=False)
-
-    with pytest.raises(AttributeTypeError):
-        TimestampWithTimezone(timestamp=ts, offset=1, negative_utc=0)
-
-    with pytest.raises(ValueError):
-        TimestampWithTimezone(timestamp=ts, offset=1, negative_utc=True)
-
-    with pytest.raises(ValueError):
-        TimestampWithTimezone(timestamp=ts, offset=-1, negative_utc=True)
+        TimestampWithTimezone(timestamp=ts, offset_bytes=0)
 
 
 def test_timestampwithtimezone_from_datetime():
+    # Typical case
     tz = datetime.timezone(datetime.timedelta(minutes=+60))
     date = datetime.datetime(2020, 2, 27, 14, 39, 19, tzinfo=tz)
-
     tstz = TimestampWithTimezone.from_datetime(date)
-
     assert tstz == TimestampWithTimezone(
-        timestamp=Timestamp(seconds=1582810759, microseconds=0,),
-        offset=60,
-        negative_utc=False,
+        timestamp=Timestamp(
+            seconds=1582810759,
+            microseconds=0,
+        ),
+        offset_bytes=b"+0100",
+    )
+
+    # Typical case (close to epoch)
+    tz = datetime.timezone(datetime.timedelta(minutes=+60))
+    date = datetime.datetime(1970, 1, 1, 1, 0, 5, tzinfo=tz)
+    tstz = TimestampWithTimezone.from_datetime(date)
+    assert tstz == TimestampWithTimezone(
+        timestamp=Timestamp(
+            seconds=5,
+            microseconds=0,
+        ),
+        offset_bytes=b"+0100",
+    )
+
+    # non-integer number of seconds before UNIX epoch
+    date = datetime.datetime(
+        1969, 12, 31, 23, 59, 59, 100000, tzinfo=datetime.timezone.utc
+    )
+    tstz = TimestampWithTimezone.from_datetime(date)
+    assert tstz == TimestampWithTimezone(
+        timestamp=Timestamp(
+            seconds=-1,
+            microseconds=100000,
+        ),
+        offset_bytes=b"+0000",
+    )
+
+    # non-integer number of seconds in both the timestamp and the offset
+    tz = datetime.timezone(datetime.timedelta(microseconds=-600000))
+    date = datetime.datetime(1969, 12, 31, 23, 59, 59, 600000, tzinfo=tz)
+    tstz = TimestampWithTimezone.from_datetime(date)
+    assert tstz == TimestampWithTimezone(
+        timestamp=Timestamp(
+            seconds=0,
+            microseconds=200000,
+        ),
+        offset_bytes=b"+0000",
+    )
+
+    # timezone offset with non-integer number of seconds, for dates before epoch
+    # we round down to the previous second, so it should be the same as
+    # 1969-01-01T23:59:59Z
+    tz = datetime.timezone(datetime.timedelta(microseconds=900000))
+    date = datetime.datetime(1970, 1, 1, 0, 0, 0, tzinfo=tz)
+    tstz = TimestampWithTimezone.from_datetime(date)
+    assert tstz == TimestampWithTimezone(
+        timestamp=Timestamp(
+            seconds=-1,
+            microseconds=100000,
+        ),
+        offset_bytes=b"+0000",
     )
 
 
@@ -515,9 +600,11 @@ def test_timestampwithtimezone_from_iso8601():
     tstz = TimestampWithTimezone.from_iso8601(date)
 
     assert tstz == TimestampWithTimezone(
-        timestamp=Timestamp(seconds=1582810759, microseconds=123456,),
-        offset=60,
-        negative_utc=False,
+        timestamp=Timestamp(
+            seconds=1582810759,
+            microseconds=123456,
+        ),
+        offset_bytes=b"+0100",
     )
 
 
@@ -527,9 +614,11 @@ def test_timestampwithtimezone_from_iso8601_negative_utc():
     tstz = TimestampWithTimezone.from_iso8601(date)
 
     assert tstz == TimestampWithTimezone(
-        timestamp=Timestamp(seconds=1582810759, microseconds=0,),
-        offset=0,
-        negative_utc=True,
+        timestamp=Timestamp(
+            seconds=1582810759,
+            microseconds=0,
+        ),
+        offset_bytes=b"-0000",
     )
 
 
@@ -545,21 +634,23 @@ def test_timestampwithtimezone_to_datetime(date, tz, microsecond):
 
 
 def test_person_from_fullname():
-    """The author should have name, email and fullname filled.
-
-    """
+    """The author should have name, email and fullname filled."""
     actual_person = Person.from_fullname(b"tony <ynot@dagobah>")
     assert actual_person == Person(
-        fullname=b"tony <ynot@dagobah>", name=b"tony", email=b"ynot@dagobah",
+        fullname=b"tony <ynot@dagobah>",
+        name=b"tony",
+        email=b"ynot@dagobah",
     )
 
 
 def test_person_from_fullname_no_email():
-    """The author and fullname should be the same as the input (author).
-
-    """
+    """The author and fullname should be the same as the input (author)."""
     actual_person = Person.from_fullname(b"tony")
-    assert actual_person == Person(fullname=b"tony", name=b"tony", email=None,)
+    assert actual_person == Person(
+        fullname=b"tony",
+        name=b"tony",
+        email=None,
+    )
 
 
 def test_person_from_fullname_empty_person():
@@ -568,7 +659,11 @@ def test_person_from_fullname_empty_person():
 
     """
     actual_person = Person.from_fullname(b"")
-    assert actual_person == Person(fullname=b"", name=None, email=None,)
+    assert actual_person == Person(
+        fullname=b"",
+        name=None,
+        email=None,
+    )
 
 
 def test_git_author_line_to_author():
@@ -577,9 +672,15 @@ def test_git_author_line_to_author():
         Person.from_fullname(None)
 
     tests = {
-        b"a <b@c.com>": Person(name=b"a", email=b"b@c.com", fullname=b"a <b@c.com>",),
+        b"a <b@c.com>": Person(
+            name=b"a",
+            email=b"b@c.com",
+            fullname=b"a <b@c.com>",
+        ),
         b"<foo@bar.com>": Person(
-            name=None, email=b"foo@bar.com", fullname=b"<foo@bar.com>",
+            name=None,
+            email=b"foo@bar.com",
+            fullname=b"<foo@bar.com>",
         ),
         b"malformed <email": Person(
             name=b"malformed", email=b"email", fullname=b"malformed <email"
@@ -590,13 +691,25 @@ def test_git_author_line_to_author():
             fullname=b'malformed <"<br"@ckets>',
         ),
         b"trailing <sp@c.e> ": Person(
-            name=b"trailing", email=b"sp@c.e", fullname=b"trailing <sp@c.e> ",
+            name=b"trailing",
+            email=b"sp@c.e",
+            fullname=b"trailing <sp@c.e> ",
+        ),
+        b"no<sp@c.e>": Person(
+            name=b"no",
+            email=b"sp@c.e",
+            fullname=b"no<sp@c.e>",
         ),
-        b"no<sp@c.e>": Person(name=b"no", email=b"sp@c.e", fullname=b"no<sp@c.e>",),
         b" more   <sp@c.es>": Person(
-            name=b"more", email=b"sp@c.es", fullname=b" more   <sp@c.es>",
+            name=b"more",
+            email=b"sp@c.es",
+            fullname=b" more   <sp@c.es>",
+        ),
+        b" <>": Person(
+            name=None,
+            email=None,
+            fullname=b" <>",
         ),
-        b" <>": Person(name=None, email=None, fullname=b" <>",),
     }
 
     for person in sorted(tests):
@@ -604,6 +717,16 @@ def test_git_author_line_to_author():
         assert expected_person == Person.from_fullname(person)
 
 
+def test_person_comparison():
+    """Check only the fullname attribute is used to compare Person objects"""
+    person = Person(fullname=b"p1", name=None, email=None)
+    assert attr.evolve(person, name=b"toto") == person
+    assert attr.evolve(person, email=b"toto@example.com") == person
+
+    person = Person(fullname=b"", name=b"toto", email=b"toto@example.com")
+    assert attr.evolve(person, fullname=b"dude") != person
+
+
 # Content
 
 
@@ -713,10 +836,19 @@ def test_content_naive_datetime():
     c = Content.from_data(b"foo")
     with pytest.raises(ValueError, match="must be a timezone-aware datetime"):
         Content(
-            **c.to_dict(), ctime=datetime.datetime.now(),
+            **c.to_dict(),
+            ctime=datetime.datetime.now(),
         )
 
 
+@given(strategies.present_contents())
+def test_content_git_roundtrip(content):
+    assert content.data is not None
+    raw = swh.model.git_objects.content_git_object(content)
+    sha1_git = hashlib.new("sha1", raw).digest()
+    assert content.sha1_git == sha1_git
+
+
 # SkippedContent
 
 
@@ -746,14 +878,15 @@ def test_skipped_content_naive_datetime():
     c = SkippedContent.from_data(b"foo", reason="reason")
     with pytest.raises(ValueError, match="must be a timezone-aware datetime"):
         SkippedContent(
-            **c.to_dict(), ctime=datetime.datetime.now(),
+            **c.to_dict(),
+            ctime=datetime.datetime.now(),
         )
 
 
 # Directory
 
 
-@given(strategies.directories().filter(lambda d: d.raw_manifest is None))
+@given(strategies.directories(raw_manifest=none()))
 def test_directory_check(directory):
     directory.check()
 
@@ -770,7 +903,7 @@ def test_directory_check(directory):
         directory2.check()
 
 
-@given(strategies.directories().filter(lambda d: d.raw_manifest is None))
+@given(strategies.directories(raw_manifest=none()))
 def test_directory_raw_manifest(directory):
     assert "raw_manifest" not in directory.to_dict()
 
@@ -810,10 +943,147 @@ def test_directory_duplicate_entry_name():
         Directory(entries=entries)
 
 
+@given(strategies.directories())
+def test_directory_from_possibly_duplicated_entries__no_duplicates(directory):
+    """
+    Directory.from_possibly_duplicated_entries should return the directory
+    unchanged if it has no duplicated entry name.
+    """
+    assert (False, directory) == Directory.from_possibly_duplicated_entries(
+        id=directory.id, entries=directory.entries, raw_manifest=directory.raw_manifest
+    )
+    assert (False, directory) == Directory.from_possibly_duplicated_entries(
+        entries=directory.entries, raw_manifest=directory.raw_manifest
+    )
+
+
+@pytest.mark.parametrize("rev_first", [True, False])
+def test_directory_from_possibly_duplicated_entries__rev_and_dir(rev_first):
+    entries = (
+        DirectoryEntry(name=b"foo", type="dir", target=b"\x01" * 20, perms=1),
+        DirectoryEntry(name=b"foo", type="rev", target=b"\x00" * 20, perms=0),
+    )
+    if rev_first:
+        entries = tuple(reversed(entries))
+    (is_corrupt, dir_) = Directory.from_possibly_duplicated_entries(entries=entries)
+    assert is_corrupt
+    assert dir_.entries == (
+        DirectoryEntry(name=b"foo", type="rev", target=b"\x00" * 20, perms=0),
+        DirectoryEntry(
+            name=b"foo_0101010101", type="dir", target=b"\x01" * 20, perms=1
+        ),
+    )
+
+    # order is independent of 'rev_first' because it is always sorted in git order
+    assert dir_.raw_manifest == (
+        # fmt: off
+        b"tree 52\x00"
+        + b"0 foo\x00" + b"\x00" * 20
+        + b"1 foo\x00" + b"\x01" * 20
+        # fmt: on
+    )
+
+
+@pytest.mark.parametrize("file_first", [True, False])
+def test_directory_from_possibly_duplicated_entries__file_and_dir(file_first):
+    entries = (
+        DirectoryEntry(name=b"foo", type="dir", target=b"\x01" * 20, perms=1),
+        DirectoryEntry(name=b"foo", type="file", target=b"\x00" * 20, perms=0),
+    )
+    if file_first:
+        entries = tuple(reversed(entries))
+    (is_corrupt, dir_) = Directory.from_possibly_duplicated_entries(entries=entries)
+    assert is_corrupt
+    assert dir_.entries == (
+        DirectoryEntry(name=b"foo", type="dir", target=b"\x01" * 20, perms=1),
+        DirectoryEntry(
+            name=b"foo_0000000000", type="file", target=b"\x00" * 20, perms=0
+        ),
+    )
+
+    # order is independent of 'file_first' because it is always sorted in git order
+    assert dir_.raw_manifest == (
+        # fmt: off
+        b"tree 52\x00"
+        + b"0 foo\x00" + b"\x00" * 20
+        + b"1 foo\x00" + b"\x01" * 20
+        # fmt: on
+    )
+
+
+def test_directory_from_possibly_duplicated_entries__two_files1():
+    entries = (
+        DirectoryEntry(name=b"foo", type="file", target=b"\x01" * 20, perms=1),
+        DirectoryEntry(name=b"foo", type="file", target=b"\x00" * 20, perms=0),
+    )
+    (is_corrupt, dir_) = Directory.from_possibly_duplicated_entries(entries=entries)
+    assert is_corrupt
+
+    assert dir_.entries == (
+        DirectoryEntry(name=b"foo", type="file", target=b"\x01" * 20, perms=1),
+        DirectoryEntry(
+            name=b"foo_0000000000", type="file", target=b"\x00" * 20, perms=0
+        ),
+    )
+    assert dir_.raw_manifest == (
+        # fmt: off
+        b"tree 52\x00"
+        + b"1 foo\x00" + b"\x01" * 20
+        + b"0 foo\x00" + b"\x00" * 20
+        # fmt: on
+    )
+
+
+def test_directory_from_possibly_duplicated_entries__two_files2():
+    """
+    Same as above, but entries are in a different order (and order matters
+    to break the tie)
+    """
+    entries = (
+        DirectoryEntry(name=b"foo", type="file", target=b"\x00" * 20, perms=0),
+        DirectoryEntry(name=b"foo", type="file", target=b"\x01" * 20, perms=1),
+    )
+    (is_corrupt, dir_) = Directory.from_possibly_duplicated_entries(entries=entries)
+    assert is_corrupt
+
+    assert dir_.entries == (
+        DirectoryEntry(name=b"foo", type="file", target=b"\x00" * 20, perms=0),
+        DirectoryEntry(
+            name=b"foo_0101010101", type="file", target=b"\x01" * 20, perms=1
+        ),
+    )
+    assert dir_.raw_manifest == (
+        # fmt: off
+        b"tree 52\x00"
+        + b"0 foo\x00" + b"\x00" * 20
+        + b"1 foo\x00" + b"\x01" * 20
+        # fmt: on
+    )
+
+
+def test_directory_from_possibly_duplicated_entries__preserve_manifest():
+    entries = (
+        DirectoryEntry(name=b"foo", type="dir", target=b"\x01" * 20, perms=1),
+        DirectoryEntry(name=b"foo", type="rev", target=b"\x00" * 20, perms=0),
+    )
+    (is_corrupt, dir_) = Directory.from_possibly_duplicated_entries(
+        entries=entries, raw_manifest=b"blah"
+    )
+    assert is_corrupt
+    assert dir_.entries == (
+        DirectoryEntry(name=b"foo", type="rev", target=b"\x00" * 20, perms=0),
+        DirectoryEntry(
+            name=b"foo_0101010101", type="dir", target=b"\x01" * 20, perms=1
+        ),
+    )
+
+    assert dir_.raw_manifest == b"blah"
+
+
 # Release
 
 
-@given(strategies.releases().filter(lambda rel: rel.raw_manifest is None))
+@given(strategies.releases(raw_manifest=none()))
 def test_release_check(release):
     release.check()
 
@@ -830,7 +1100,7 @@ def test_release_check(release):
         release2.check()
 
 
-@given(strategies.releases().filter(lambda rev: rev.raw_manifest is None))
+@given(strategies.releases(raw_manifest=none()))
 def test_release_raw_manifest(release):
     raw_manifest = b"foo"
     id_ = hashlib.new("sha1", raw_manifest).digest()
@@ -850,7 +1120,7 @@ def test_release_raw_manifest(release):
 # Revision
 
 
-@given(strategies.revisions().filter(lambda rev: rev.raw_manifest is None))
+@given(strategies.revisions(raw_manifest=none()))
 def test_revision_check(revision):
     revision.check()
 
@@ -867,7 +1137,7 @@ def test_revision_check(revision):
         revision2.check()
 
 
-@given(strategies.revisions().filter(lambda rev: rev.raw_manifest is None))
+@given(strategies.revisions(raw_manifest=none()))
 def test_revision_raw_manifest(revision):
 
     raw_manifest = b"foo"
@@ -1070,6 +1340,33 @@ def test_revision_extra_headers_as_lists_from_dict():
     assert rev_model.extra_headers == extra_headers
 
 
+def test_revision_no_author_or_committer_from_dict():
+    rev_dict = revision_example.copy()
+    rev_dict["author"] = rev_dict["date"] = None
+    rev_dict["committer"] = rev_dict["committer_date"] = None
+    rev_model = Revision.from_dict(rev_dict)
+    assert rev_model.to_dict() == {
+        **rev_dict,
+        "parents": tuple(rev_dict["parents"]),
+        "extra_headers": (),
+        "metadata": None,
+    }
+
+
+def test_revision_none_author_or_committer():
+    rev_dict = revision_example.copy()
+    rev_dict["author"] = None
+    with pytest.raises(ValueError, match=".*date must be None if author is None.*"):
+        Revision.from_dict(rev_dict)
+
+    rev_dict = revision_example.copy()
+    rev_dict["committer"] = None
+    with pytest.raises(
+        ValueError, match=".*committer_date must be None if committer is None.*"
+    ):
+        Revision.from_dict(rev_dict)
+
+
 @given(strategies.objects(split_content=True))
 def test_object_type(objtype_and_obj):
     obj_type, obj = objtype_and_obj
@@ -1092,15 +1389,19 @@ def test_object_type_is_final():
 
 
 _metadata_authority = MetadataAuthority(
-    type=MetadataAuthorityType.FORGE, url="https://forge.softwareheritage.org",
+    type=MetadataAuthorityType.FORGE,
+    url="https://forge.softwareheritage.org",
+)
+_metadata_fetcher = MetadataFetcher(
+    name="test-fetcher",
+    version="0.0.1",
 )
-_metadata_fetcher = MetadataFetcher(name="test-fetcher", version="0.0.1",)
 _content_swhid = ExtendedSWHID.from_string(
     "swh:1:cnt:94a9ed024d3859793618152ea559a168bbcbb5e2"
 )
 _origin_url = "https://forge.softwareheritage.org/source/swh-model.git"
 _origin_swhid = ExtendedSWHID.from_string(
-    "swh:1:ori:94a9ed024d3859793618152ea559a168bbcbb5e2"
+    "swh:1:ori:433b4f5612f0720ed51fa7aeaf43a3625870057b"
 )
 _dummy_qualifiers = {"origin": "https://example.com", "lines": "42"}
 _common_metadata_fields = dict(
@@ -1122,7 +1423,43 @@ def test_metadata_valid():
 
     # Object with an SWHID
     RawExtrinsicMetadata(
-        target=_content_swhid, **_common_metadata_fields,
+        target=_content_swhid,
+        **_common_metadata_fields,
+    )
+
+
+def test_metadata_from_old_dict():
+    common_fields = {
+        "authority": {"type": "forge", "url": "https://forge.softwareheritage.org"},
+        "fetcher": {
+            "name": "test-fetcher",
+            "version": "0.0.1",
+        },
+        "discovery_date": _common_metadata_fields["discovery_date"],
+        "format": "json",
+        "metadata": b'{"origin": "https://example.com", "lines": "42"}',
+    }
+
+    m = RawExtrinsicMetadata(
+        target=_origin_swhid,
+        **_common_metadata_fields,
+    )
+    assert (
+        RawExtrinsicMetadata.from_dict(
+            {"id": m.id, "target": _origin_url, "type": "origin", **common_fields}
+        )
+        == m
+    )
+
+    m = RawExtrinsicMetadata(
+        target=_content_swhid,
+        **_common_metadata_fields,
+    )
+    assert (
+        RawExtrinsicMetadata.from_dict(
+            {"target": str(_content_swhid), "type": "content", **common_fields}
+        )
+        == m
     )
 
 
@@ -1131,21 +1468,30 @@ def test_metadata_to_dict():
 
     common_fields = {
         "authority": {"type": "forge", "url": "https://forge.softwareheritage.org"},
-        "fetcher": {"name": "test-fetcher", "version": "0.0.1",},
+        "fetcher": {
+            "name": "test-fetcher",
+            "version": "0.0.1",
+        },
         "discovery_date": _common_metadata_fields["discovery_date"],
         "format": "json",
         "metadata": b'{"origin": "https://example.com", "lines": "42"}',
     }
 
-    m = RawExtrinsicMetadata(target=_origin_swhid, **_common_metadata_fields,)
+    m = RawExtrinsicMetadata(
+        target=_origin_swhid,
+        **_common_metadata_fields,
+    )
     assert m.to_dict() == {
         "target": str(_origin_swhid),
-        "id": b"@j\xc9\x01\xbc\x1e#p*\xf3q9\xa7u\x97\x00\x14\x02xa",
+        "id": b"\xa3)q\x0f\xf7p\xc7\xb0\\O\xe8\x84\x83Z\xb0]\x81\xe9\x95\x13",
         **common_fields,
     }
     assert RawExtrinsicMetadata.from_dict(m.to_dict()) == m
 
-    m = RawExtrinsicMetadata(target=_content_swhid, **_common_metadata_fields,)
+    m = RawExtrinsicMetadata(
+        target=_content_swhid,
+        **_common_metadata_fields,
+    )
     assert m.to_dict() == {
         "target": "swh:1:cnt:94a9ed024d3859793618152ea559a168bbcbb5e2",
         "id": b"\xbc\xa3U\xddf\x19U\xc5\xd2\xd7\xdfK\xd7c\x1f\xa8\xfeh\x992",
@@ -1205,12 +1551,16 @@ def test_metadata_validate_context_origin():
         ValueError, match="Unexpected 'origin' context for origin object"
     ):
         RawExtrinsicMetadata(
-            target=_origin_swhid, origin=_origin_url, **_common_metadata_fields,
+            target=_origin_swhid,
+            origin=_origin_url,
+            **_common_metadata_fields,
         )
 
     # but all other types can
     RawExtrinsicMetadata(
-        target=_content_swhid, origin=_origin_url, **_common_metadata_fields,
+        target=_content_swhid,
+        origin=_origin_url,
+        **_common_metadata_fields,
     )
 
     # SWHIDs aren't valid origin URLs
@@ -1230,18 +1580,25 @@ def test_metadata_validate_context_visit():
         ValueError, match="Unexpected 'visit' context for origin object"
     ):
         RawExtrinsicMetadata(
-            target=_origin_swhid, visit=42, **_common_metadata_fields,
+            target=_origin_swhid,
+            visit=42,
+            **_common_metadata_fields,
         )
 
     # but all other types can
     RawExtrinsicMetadata(
-        target=_content_swhid, origin=_origin_url, visit=42, **_common_metadata_fields,
+        target=_content_swhid,
+        origin=_origin_url,
+        visit=42,
+        **_common_metadata_fields,
     )
 
     # Missing 'origin'
     with pytest.raises(ValueError, match="'origin' context must be set if 'visit' is"):
         RawExtrinsicMetadata(
-            target=_content_swhid, visit=42, **_common_metadata_fields,
+            target=_content_swhid,
+            visit=42,
+            **_common_metadata_fields,
         )
 
     # visit id must be positive
@@ -1264,7 +1621,8 @@ def test_metadata_validate_context_snapshot():
         RawExtrinsicMetadata(
             target=_origin_swhid,
             snapshot=CoreSWHID(
-                object_type=ObjectType.SNAPSHOT, object_id=EXAMPLE_HASH,
+                object_type=ObjectType.SNAPSHOT,
+                object_id=EXAMPLE_HASH,
             ),
             **_common_metadata_fields,
         )
@@ -1282,7 +1640,10 @@ def test_metadata_validate_context_snapshot():
     ):
         RawExtrinsicMetadata(
             target=_content_swhid,
-            snapshot=CoreSWHID(object_type=ObjectType.CONTENT, object_id=EXAMPLE_HASH,),
+            snapshot=CoreSWHID(
+                object_type=ObjectType.CONTENT,
+                object_id=EXAMPLE_HASH,
+            ),
             **_common_metadata_fields,
         )
 
@@ -1296,7 +1657,10 @@ def test_metadata_validate_context_release():
     ):
         RawExtrinsicMetadata(
             target=_origin_swhid,
-            release=CoreSWHID(object_type=ObjectType.RELEASE, object_id=EXAMPLE_HASH,),
+            release=CoreSWHID(
+                object_type=ObjectType.RELEASE,
+                object_id=EXAMPLE_HASH,
+            ),
             **_common_metadata_fields,
         )
 
@@ -1313,7 +1677,10 @@ def test_metadata_validate_context_release():
     ):
         RawExtrinsicMetadata(
             target=_content_swhid,
-            release=CoreSWHID(object_type=ObjectType.CONTENT, object_id=EXAMPLE_HASH,),
+            release=CoreSWHID(
+                object_type=ObjectType.CONTENT,
+                object_id=EXAMPLE_HASH,
+            ),
             **_common_metadata_fields,
         )
 
@@ -1328,7 +1695,8 @@ def test_metadata_validate_context_revision():
         RawExtrinsicMetadata(
             target=_origin_swhid,
             revision=CoreSWHID(
-                object_type=ObjectType.REVISION, object_id=EXAMPLE_HASH,
+                object_type=ObjectType.REVISION,
+                object_id=EXAMPLE_HASH,
             ),
             **_common_metadata_fields,
         )
@@ -1346,7 +1714,10 @@ def test_metadata_validate_context_revision():
     ):
         RawExtrinsicMetadata(
             target=_content_swhid,
-            revision=CoreSWHID(object_type=ObjectType.CONTENT, object_id=EXAMPLE_HASH,),
+            revision=CoreSWHID(
+                object_type=ObjectType.CONTENT,
+                object_id=EXAMPLE_HASH,
+            ),
             **_common_metadata_fields,
         )
 
@@ -1357,12 +1728,16 @@ def test_metadata_validate_context_path():
     # Origins can't have a 'path' context
     with pytest.raises(ValueError, match="Unexpected 'path' context for origin object"):
         RawExtrinsicMetadata(
-            target=_origin_swhid, path=b"/foo/bar", **_common_metadata_fields,
+            target=_origin_swhid,
+            path=b"/foo/bar",
+            **_common_metadata_fields,
         )
 
     # but content can
     RawExtrinsicMetadata(
-        target=_content_swhid, path=b"/foo/bar", **_common_metadata_fields,
+        target=_content_swhid,
+        path=b"/foo/bar",
+        **_common_metadata_fields,
     )
 
 
@@ -1376,7 +1751,8 @@ def test_metadata_validate_context_directory():
         RawExtrinsicMetadata(
             target=_origin_swhid,
             directory=CoreSWHID(
-                object_type=ObjectType.DIRECTORY, object_id=EXAMPLE_HASH,
+                object_type=ObjectType.DIRECTORY,
+                object_id=EXAMPLE_HASH,
             ),
             **_common_metadata_fields,
         )
@@ -1384,7 +1760,10 @@ def test_metadata_validate_context_directory():
     # but content can
     RawExtrinsicMetadata(
         target=_content_swhid,
-        directory=CoreSWHID(object_type=ObjectType.DIRECTORY, object_id=EXAMPLE_HASH,),
+        directory=CoreSWHID(
+            object_type=ObjectType.DIRECTORY,
+            object_id=EXAMPLE_HASH,
+        ),
         **_common_metadata_fields,
     )
 
@@ -1395,7 +1774,8 @@ def test_metadata_validate_context_directory():
         RawExtrinsicMetadata(
             target=_content_swhid,
             directory=CoreSWHID(
-                object_type=ObjectType.CONTENT, object_id=EXAMPLE_HASH,
+                object_type=ObjectType.CONTENT,
+                object_id=EXAMPLE_HASH,
             ),
             **_common_metadata_fields,
         )
@@ -1416,7 +1796,9 @@ def test_metadata_normalize_discovery_date():
     # Check for truncation to integral second
     date_with_us = truncated_date.replace(microsecond=42)
     md = RawExtrinsicMetadata(
-        target=_content_swhid, discovery_date=date_with_us, **fields_copy,
+        target=_content_swhid,
+        discovery_date=date_with_us,
+        **fields_copy,
     )
 
     assert md.discovery_date == truncated_date
@@ -1430,7 +1812,9 @@ def test_metadata_normalize_discovery_date():
     assert date_with_tz.tzinfo != datetime.timezone.utc
 
     md = RawExtrinsicMetadata(
-        target=_content_swhid, discovery_date=date_with_tz, **fields_copy,
+        target=_content_swhid,
+        discovery_date=date_with_tz,
+        **fields_copy,
     )
 
     assert md.discovery_date == truncated_date
